@@ -17,6 +17,8 @@
   License along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "config.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -34,16 +36,16 @@ struct entry_handle {
 	dev_t dev;
 	ino_t ino;
 	DIR *stream;
-	off_t pos;
+	long pos;
 };
 
-struct entry_handle head = {
+static struct entry_handle head = {
 	.next = &head,
 	.prev = &head,
 	/* The other fields are unused. */
 };
-struct entry_handle *closed = &head;
-unsigned int num_dir_handles;
+static struct entry_handle *closed = &head;
+static unsigned int num_dir_handles;
 
 static int walk_tree_visited(dev_t dev, ino_t ino)
 {
@@ -60,7 +62,8 @@ static int walk_tree_rec(const char *path, int walk_flags,
 				     void *), void *arg, int depth)
 {
 	int follow_symlinks = (walk_flags & WALK_TREE_LOGICAL) ||
-			      (!(walk_flags & WALK_TREE_PHYSICAL) &&
+			      ((walk_flags & WALK_TREE_DEREFERENCE) &&
+			       !(walk_flags & WALK_TREE_PHYSICAL) &&
 			       depth == 0);
 	int have_dir_stat = 0, flags = walk_flags, err;
 	struct entry_handle dir;
@@ -78,7 +81,9 @@ static int walk_tree_rec(const char *path, int walk_flags,
 		return func(path, NULL, flags | WALK_TREE_FAILED, arg);
 	if (S_ISLNK(st.st_mode)) {
 		flags |= WALK_TREE_SYMLINK;
-		if (flags & WALK_TREE_DEREFERENCE) {
+		if ((flags & WALK_TREE_DEREFERENCE) ||
+		    ((flags & WALK_TREE_TOPLEVEL) &&
+		     (flags & WALK_TREE_DEREFERENCE_TOPLEVEL))) {
 			if (stat(path, &st) != 0)
 				return func(path, NULL,
 					    flags | WALK_TREE_FAILED, arg);
@@ -99,8 +104,8 @@ static int walk_tree_rec(const char *path, int walk_flags,
 	 *      a link and follow_symlinks
 	 */
         if ((flags & WALK_TREE_RECURSIVE) &&
-	   (!(flags & WALK_TREE_SYMLINK) && S_ISDIR(st.st_mode)) ||
-	   ((flags & WALK_TREE_SYMLINK) && follow_symlinks)) {
+	    ((!(flags & WALK_TREE_SYMLINK) && S_ISDIR(st.st_mode)) ||
+	     ((flags & WALK_TREE_SYMLINK) && follow_symlinks))) {
 		struct dirent *entry;
 
 		/*
